package org.appfuse.aop;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class CacheInterceptor implements MethodInterceptor {
	private final Log log = LogFactory.getLog(CacheInterceptor.class);
    
    public Object invoke(MethodInvocation invocation) throws Throwable {
        String name = invocation.getMethod().getName();
        Object returnValue;
        
        // check cache before executing method
        if (name.indexOf("get") > -1 && !name.endsWith("s")) {
            String id = (String) invocation.getArguments()[0];
            returnValue = cache.get(id);
            if (returnValue == null) {
                // user not in cache, proceed
                returnValue = invocation.proceed();
                putIntoCache(id, returnValue);
                return returnValue;
            } else {
                //log.debug("retrieved object id '" + id + "' from cache");
            }
        } else {
            returnValue = invocation.proceed();
       
            // update cache after executing method
            if (name.indexOf("save") > -1) {
                Method getId = returnValue.getClass().getMethod("getId", new Class[]{});
                Long id = (Long) getId.invoke(returnValue, new Object[]{});
                putIntoCache(String.valueOf(id), returnValue);
            } else if (name.indexOf("remove") > -1) {
                String id = (String) invocation.getArguments()[0];
                removeFromCache(String.valueOf(id));
            }
        }
        return returnValue;
    }

    protected Map cache;
    
    protected void putIntoCache(String key, Object value) {
         if (cache == null) {
            cache = new HashMap();
         }
         cache.put(key, value);
    }
    
    protected void removeFromCache(String key) {
        if (cache != null) {
            cache.remove(key);
        }
    }
}
