package org.appfuse.service;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.appfuse.model.User;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.dao.DataAccessException;
import org.springframework.util.StopWatch;

public class UserManagerTest extends TestCase {
    private static Log log = LogFactory.getLog(UserManagerTest.class);
    private ApplicationContext ctx;
    private User user = null;
    private UserManager mgr = null;

    protected void setUp() throws Exception {
        String[] paths = {"/WEB-INF/applicationContext*.xml"};
        ctx = new ClassPathXmlApplicationContext(paths);
        mgr = (UserManager) ctx.getBean("userManager");
        
        // Modify the mailSender bean to use Dumbster's ports
        /*JavaMailSenderImpl mailSender = 
            (JavaMailSenderImpl) ctx.getBean("mailSender");
        mailSender.setPort(2525);*/
        
    }
    
    public void testAddAndRemoveUser() throws Exception {
        user = new User();
        user.setFirstName("Easter");
        user.setLastName("Bunny");

        // setup a simple mail server using Dumbster
        //SimpleSmtpServer server = SimpleSmtpServer.start(2525);
        
        user = mgr.saveUser(user);

        /*server.stop();
        assertEquals(1, server.getReceievedEmailSize()); // spelling is correct
        SmtpMessage sentMessage = (SmtpMessage) server.getReceivedEmail().next();
        assertTrue(sentMessage.getBody().indexOf("Easter Bunny") != -1);
        log.debug(sentMessage);*/
        
        assertTrue(user.getId() != null);

        if (log.isDebugEnabled()) {
            log.debug("removing user...");
        }

        String userId = user.getId().toString();
        mgr.removeUser(userId);

        try {
            user = mgr.getUser(userId);
            fail("User '" + userId + "' found in database");
        } catch (DataAccessException dae) {
            log.debug("Expected exception: " + dae.getMessage());
            assertTrue(dae != null);
        }
    }

    public void testGetUserPerformance() {
        user = new User();
        user.setFirstName("Easter");
        user.setLastName("Bunny");

        user = mgr.saveUser(user);
        
        String name = "getUser";
        StopWatch sw = new StopWatch(name);
        sw.start(name);
        log.debug("Begin timing of method '" + name + "'");
        
        for (int i=0; i < 200; i++) {
            mgr.getUser(user.getId().toString());
        }
        
        sw.stop();
        log.info(sw.shortSummary());
        log.debug("End timing of method '" + name + "'");
    }
    /*
    public void testWithValidationErrors() {
        user = new User();
        user.setFirstName("Bill");
        
        try {
            user = mgr.saveUser(user);
            fail("Validation exception not thrown!");
        } catch (Exception e) {
            log.debug(e.getCause().getMessage());
            assertNotNull(e.getCause());
        }
    }*/
}
